#
/*
 *		M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *				    D R D D I R . C
 *
 *				Douglas P. Kingston III
 *				  Michael John Muuss
 *
 *				Part of the MDQS Daemon
 *
 *				   Revision History
 *				   ================
 *	13 Sep 82  DPK	Added "Waveoff" related code to prevent directory
 *			searches when we know there is no space for the
 *			requests.
 *
 *	20 Dec 82  DPK	Modified to use libndir.a directory access routines.
 */

#include <stdio.h>
#include "queue.h"		/* Includes <sys/types.h> */
#include "dirent.h"
#include <sys/stat.h>

#ifdef DIRBLKSIZ		/* 4BSD flavor assumed */
#define	dirent	direct
#endif

extern	char *Newprefix;
extern	char *Modprefix;
extern	int Waveoff;


rdnew ()
{
	register DIR	*df;
	register struct dirent *dp;

	if(( df = opendir (Newprefix)) == NULL ) {
		complain ("rdnew(): %s: %s", Newprefix, sys_errlist[errno]);
		return;
	}

	while (( dp = readdir( df )) != NULL ) {
		/*
		 *  Request must be new!
		 */
		if(dp->d_name[0] == '.')
			continue;	/* skip files that begin with . */
		enroll (dp->d_name);
		if( Waveoff )
			break;
		rmreq (Newprefix, dp->d_name, NOISY);
	}
	closedir (df);
}


rdmod()
{
	register DIR	*df;
	register struct dirent *dp;

	if(( df = opendir (Modprefix)) == NULL ) {
		complain ("rdmod(): %s: %s", Modprefix, sys_errlist[errno]);
		return;
	}

	while (( dp = readdir( df )) != NULL ) {
		/*
		 *  This must be a modified request
		 */
		if(dp->d_name[0] == '.')
			continue;	/* skip files that begin with . */
		update (dp->d_name);
		rmreq (Modprefix, dp->d_name, NOISY);
	}
	closedir (df);
}

rmreq ( prefix, rname, noisy )
register char	*prefix;
register char	*rname;
int	noisy;
{
	struct	stat sb;
	char	namebuf[PATHSIZ];

	strcpy (namebuf, prefix);
	strcat (namebuf, rname);
	if (stat (namebuf, &sb) == 0 && (sb.st_mode&S_IFMT) == S_IFDIR)
		return;
	if ( unlink (namebuf) < 0 && noisy )
		complain ("Rmreq unlink failed (%s: %s)", namebuf,
			  sys_errlist[errno]);
}
