/*
 *			Q D E V . C 
 *
 * $Revision: 1.2 $
 *
 * $Log:	qdev.c,v $
 * Revision 1.2  85/12/26  17:29:54  tcs
 * Added "Usage:..." lines on error conditions.
 * 
 * Revision 1.3  84/02/29  19:06:52  dpk
 * Added DPW device status stuff using RCS merge....
 * 
 * Revision 1.2  84/02/29  18:35:33  dpk
 * Added distinctive RCS header
 * 
 */
#ifndef lint
static char RCSid[] = "@(#)$Header: qdev.c,v 1.2 85/12/26 17:29:54 tcs BRL $";
#endif

#
/*
 *	    M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *			     Douglas P. Kingston III
 *
 *			 Device control program  (QDEV)
 *
 *				Revision History
 *				================
 */
#include <stdio.h>
#include <signal.h>
#include "queue.h"
#include "devstat.h"

extern	char	*Qcntrldir;
extern	char	*Admprefix;
extern	char	*Qconffile;
extern	lock_t	Dae_lock;
extern	lock_t	Dev_lock;

extern	char	*ckforms();
extern	char	*strdup();

char	**getdevs();

#define	DISABLE	1
#define	ENABLE	2
#define	FLUSH	3
#define	LOAD	4
#define	RESTART	5
#define	STATUS	6

main( argc, argv )
int	argc;
register char	**argv;
{
	register FILE	*pf;
	register int	func;
	int	update = 0;
	char	*forms = NULL;
	char	dsfile[PATHSIZ];
	struct	devstat ds;
	char	statmsg[64];
	int	nread;

	fdverify();			/* Paranoid */

	if(argc < 1) {
		err( "qdev: bad invocation (argc < 1)" );
		err( "Usage:  qdev [edf[l form]rs] device [device ...]" );
		exit( 9 );
	}

	if( chdir( Qcntrldir ) < 0 ) {
		err( "Can't chdir to %s.", Qcntrldir );
		err( "Please inform system management." );
		exit( 1 );
	}

	argv++;
	if(*argv && **argv == '-')
		(*argv)++;

	if( *argv == 0 )
		func = STATUS;
	else if( strcmp(*argv, "d") == SAME )
		update++, func = DISABLE;
	else if( strcmp(*argv, "e") == SAME )
		update++, func = ENABLE;
	else if( strcmp(*argv, "f") == SAME )
		func = FLUSH;
	else if( strcmp(*argv, "l") == SAME )
		update++, func = LOAD;
	else if( strcmp(*argv, "r") == SAME )
		func = RESTART;
	else if( strcmp(*argv, "s") == SAME )
		func = STATUS;
	else if( strcmp(*argv, "?") == SAME ) {
		err( "Usage:  qdev [edf[l form]rs] device [device ...]" );
		exit( 1 );
	} else {
		err("Unknown function '%s'", *argv);
		err( "Usage:  qdev [edf[l form]rs] device [device ...]" );
		exit(9);
	}

	if (*argv)
		argv++;
	if( func == LOAD ) {
		if (*argv == 0) {
			err("Missing form name");
			exit(8);
		}
		if ((forms = ckforms(*argv)) == NULL) {
			err("Unknown form '%s'", *argv);
			exit(8);
		}
		argv++;
	} else if( func == STATUS && *argv == NULL )
		argv = getdevs();

	if( lock( &Dae_lock, NOWAIT ) == 0 ) {
		fputs("The daemon does not appear to be running.\n", stdout);
		unlock( &Dae_lock );
	}

	if (update)
		if( lock(&Dev_lock, NOWAIT) != 0 ) {
			err( "Device update lock unavailable, try again.");
			exit(1);
		}

	if ( func == FLUSH )
		if(!mdqsmanager())
			setuid( getuid() );

	while( *argv ) {
		sprintf(dsfile, "%s/D_%s", Admprefix, *argv );
		if(( pf = fopen( dsfile, (update ? "r+" : "r") )) == NULL ) {
			err( "Device %s not configured", *argv );
			argv++;
			continue;
		} else {
			if (fread(&ds, sizeof ds, 1, pf) != 1) {
				err("Error reading device status of %s", *argv);
				argv++;
				fclose( pf );
				continue;
			}

			switch( func ) {
			case ENABLE:
				ds.uds.uds_status &= ~D_DISABLED;
				dump(*argv, pf, &ds.uds);
				break;
			case FLUSH:
				if( ds.dds.dds_pid == 0 )
					err( "Device %s not active", *argv);
				else if( kill( ds.dds.dds_pid, SIGTERM ) < 0 )
					err( "No permisson to flush %s",
						ds.dds.dds_rname );
				else
					printf("%s flushed\n",
						ds.dds.dds_rname);
				break;
			case LOAD:
				strncpy( ds.uds.uds_forms, forms, FORMSIZ );
				dump(*argv, pf, &ds.uds);
				break;
			case DISABLE:
				ds.uds.uds_status |= D_DISABLED;
				dump(*argv, pf, &ds.uds);
				if(ds.dds.dds_pid == 0)
					break;
				/* FALL THROUGH */
			case RESTART:
				if( ds.dds.dds_pid == 0 )
					err( "Device %s not active", *argv);
				else if( kill( ds.dds.dds_pid, SIGEMT ) < 0 )
					err( "No permisson to restart %s",
						ds.dds.dds_rname );
				else
					printf("%s restarted\n",
						ds.dds.dds_rname);
				break;
			case STATUS:
				printf("%-*.*s %-*.*s", QDIRSIZ, QDIRSIZ,
					*argv, FORMSIZ, FORMSIZ, ds.uds.uds_forms);
				if( ds.dds.dds_pid )
					printf(" %-*.*s  %d", QDIRSIZ,
					     QDIRSIZ, ds.dds.dds_rname, ds.dds.dds_pid);
				if(ds.dds.dds_status & D_FAILED)
					fputs("  Failed", stdout);
				if(ds.uds.uds_status & D_DISABLED)
					fputs("  Disabled", stdout);
				nread = fread (statmsg, 1, LINESIZ-1, pf);
				if (nread && statmsg[0]) {
					statmsg[nread] = 0;
					printf(" (%-.*s)", LINESIZ-1, statmsg);
				}
				putc('\n', stdout);
				break;
			}
			fclose( pf );
		}
		argv++;
	}
	if( update )
		unlock( &Dev_lock );
	proddaemon();
	exit(0);
}

dump(dname, pf, dsp)
register char *dname;
register FILE *pf;
register struct usrdevstat *dsp;
{
	rewind( pf );
	fseek(pf, (long) sizeof (struct daedevstat), 0);
	fwrite(dsp, sizeof (struct usrdevstat), 1, pf);
	fflush(pf);
	if( ferror(pf) )
		err("Update error device %s (%s)",
		     dname, sys_errlist[errno]);
}




/*
 *			GETDEVS
 *
 *	Build a list of configured devices.
 */
char **
getdevs()
{
	register int	filestate;
	register FILE	*cf;
	register int i;
	int	argcnt;
	char	linebuf[LINESIZ];
	char	**fieldp;
	static char	*devs[50];

	if(( cf = fopen (Qconffile, "r")) == NULL )
		bomb ("Cannot open configuration file (%s: %s)",
			Qconffile, sys_errlist[errno]);

	i = 0;
	filestate = 0;
	while (fgets (linebuf, LINESIZ, cf) != NULL) {

		argcnt = parse (linebuf, &fieldp);

		/*  "Blank" line or bad line, probably a comment line */
		if (argcnt <= 0)
			continue;

		/*  Lines that start with a "-" indicate end of a section */
		if (**fieldp == '-') {
			filestate++;
			continue;
		}

		if( filestate == 1 ) {
			/*
			 *  A device definition
			 */
			if( argcnt < 2 )
				continue;
			devs[i++] = strdup(*fieldp);
		} else if( filestate == 2 )
			break;
	}
	fclose( cf );
	devs[i] = NULL;
	return( devs );
}
