/*
 *		M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *				  Douglas A. Gwyn
 *
 *				  Q L O C K S . C
 *
 *				for fcntl() locking
 *
 *	-----------------------------------------------------------------------
 *
 *				Revision History
 *				================
 *
 *	15-Mar-1988	D A Gwyn	Created initial version without
 *					clearlock(), which is never used.
 */

#include	<errno.h>
#include	<fcntl.h>

extern int	close(), fcntl(), open(), umask(), write();
extern unsigned	sleep();

#include	"queue.h"		/* defines lock_t */

#define	SUCCESS	0
#define	FAILURE	-1

#undef NULL
#define	NULL	0

#define	BADFD	-1

extern	lock_t	*All_locks[];

/*
 *		L O C K  ( )
 *  Create lock on named file; keep trying for a while if already locked.
 */
int
lock( lockname, waittime )
	register lock_t	*lockname;
	int		waittime;	/* max. seconds to block */
	{
	struct flock	lock_data;	/* lock spec for fcntl() */
	int		omask = umask( 0 );	/* save original umask */

	while ( lockname->lfd == BADFD 
	     && (lockname->lfd = open( lockname->lockfile,
				       O_WRONLY | O_CREAT | O_TRUNC | O_NDELAY,
				       0666
				     )
		) == BADFD
	      )
		switch ( errno )
			{
		case EINTR:		/* interrupted by a signal */
		case ENFILE:		/* system file table temporarily full */
		case ENOSPC:		/* file system temporarily out of space */

			if ( waittime <= 0 )
				{	/* timed out */
				(void)umask( omask );
				return FAILURE;	/* timeout */
				}
			else	{	/* wait, then try again */
				(void)sleep( (unsigned)1 );
				--waittime;
				}

			break;

		default:
			err( "open error on %s (%s)",
			     lockname->lockfile, sys_errlist[errno]
			   );
			(void)umask( omask );
			return FAILURE;
			}

	(void)write( lockname->lfd, "", 1 );	/* give the file some contents */

	lock_data.l_type = F_WRLCK;	/* exclusive lock */
	lock_data.l_whence = 0;
	lock_data.l_start = 0L;
	lock_data.l_len = 0L;		/* entire file (1 whole byte) */

	while ( fcntl( lockname->lfd, F_SETLK, &lock_data ) != 0 )
		switch ( errno )
			{
		case EACCES:		/* file locked by another process */
		case ENOLCK:		/* system temporarily out of locks */
		case EINTR:		/* interrupted by a signal */

			if ( waittime <= 0 )
				{	/* timed out */
				(void)close( lockname->lfd );
				(void)umask( omask );
				return FAILURE;
				}
			else	{	/* wait, then try again */
				(void)sleep( (unsigned)1 );
				--waittime;
				}

			break;

		default:
			err( "lock error on %s (%s)",
			     lockname->lockfile, sys_errlist[errno]
			   );
			(void)close( lockname->lfd );
			(void)umask( omask );
			return FAILURE;
			}

	(void)umask( omask );
	return SUCCESS;			/* file now locked */
	}

/*
 *		U N L O C K  ( )
 *  Remove this process's lock on specified file.
 */
int					/* (return value never actually used) */
unlock( lockname )
	lock_t		*lockname;
	{
	struct flock	lock_data;	/* lock spec for fcntl() */

	if ( lockname->lfd == BADFD )
		return SUCCESS;

	lock_data.l_type = F_UNLCK;	/* remove lock */
	lock_data.l_whence = 0;
	lock_data.l_start = 0L;
	lock_data.l_len = 0L;		/* entire file (1 whole byte) */

	return fcntl( lockname->lfd, F_SETLK, &lock_data );
	}
